import { useState, useEffect, useMemo, useLayoutEffect } from 'react';
import Mousetrap from 'mousetrap';

const mousetraps = new WeakMap();
const getMousetrap = (element) => {
    if (element) {
        let mousetrap = mousetraps.get(element);
        if (!mousetrap) {
            mousetrap = Mousetrap(element);
            mousetraps.set(element, mousetrap);
        }
        return mousetrap;
    }
    return Mousetrap;
};
const shortcutMousetraps = new WeakMap();
/**
 * Creates a global state singleton.
 */
const createStateHook = () => {
    let keys = [];
    let callbacks = new Set();
    const onChanged = () => {
        for (const callback of callbacks) {
            try {
                callback(keys);
            }
            catch (e) {
                console.error(e);
            }
        }
    };
    const addKeys = (nextKeys) => {
        keys = [...keys, ...nextKeys];
        nextKeys.forEach((k) => {
            var _a;
            if (!k.disabled) {
                const mousetrap = getMousetrap((_a = k.ref) === null || _a === void 0 ? void 0 : _a.current);
                shortcutMousetraps.set(k, mousetrap);
                mousetrap.bind(k.keys, k.callback, k.action);
            }
        });
        onChanged();
    };
    const removeKeys = (nextKeys) => {
        keys = keys.filter((k) => !nextKeys.includes(k));
        nextKeys.forEach(s => { var _a; return (_a = shortcutMousetraps.get(s)) === null || _a === void 0 ? void 0 : _a.unbind(s.keys, s.action); });
        onChanged();
    };
    return () => {
        const [state, setState] = useState(keys);
        useEffect(() => {
            callbacks.add(setState);
            return () => callbacks.delete(setState);
        });
        return useMemo(() => [state, addKeys, removeKeys], [state]);
    };
};
const useHotkeyState = createStateHook();

const useHotkeys = (shortcuts) => {
    const [keys, addKeys, removeKeys] = useHotkeyState();
    useLayoutEffect(() => {
        if (shortcuts) {
            addKeys(shortcuts);
        }
        return () => {
            if (shortcuts) {
                removeKeys(shortcuts);
            }
        };
    }, [shortcuts]);
    return useMemo(() => keys.reduce((agg, cur) => {
        if (!agg.find((a) => a.keys === cur.keys && a.ref && cur.ref)) {
            agg.push(cur);
        }
        return agg;
    }, []), [keys]);
};

export { useHotkeyState, useHotkeys };
