(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('mousetrap')) :
  typeof define === 'function' && define.amd ? define(['exports', 'react', 'mousetrap'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.reakeys = {}, global.react, global.Mousetrap));
}(this, (function (exports, react, Mousetrap) { 'use strict';

  function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

  var Mousetrap__default = /*#__PURE__*/_interopDefaultLegacy(Mousetrap);

  const mousetraps = new WeakMap();
  const getMousetrap = (element) => {
      if (element) {
          let mousetrap = mousetraps.get(element);
          if (!mousetrap) {
              mousetrap = Mousetrap__default['default'](element);
              mousetraps.set(element, mousetrap);
          }
          return mousetrap;
      }
      return Mousetrap__default['default'];
  };
  const shortcutMousetraps = new WeakMap();
  /**
   * Creates a global state singleton.
   */
  const createStateHook = () => {
      let keys = [];
      let callbacks = new Set();
      const onChanged = () => {
          for (const callback of callbacks) {
              try {
                  callback(keys);
              }
              catch (e) {
                  console.error(e);
              }
          }
      };
      const addKeys = (nextKeys) => {
          keys = [...keys, ...nextKeys];
          nextKeys.forEach((k) => {
              var _a;
              if (!k.disabled) {
                  const mousetrap = getMousetrap((_a = k.ref) === null || _a === void 0 ? void 0 : _a.current);
                  shortcutMousetraps.set(k, mousetrap);
                  mousetrap.bind(k.keys, k.callback, k.action);
              }
          });
          onChanged();
      };
      const removeKeys = (nextKeys) => {
          keys = keys.filter((k) => !nextKeys.includes(k));
          nextKeys.forEach(s => { var _a; return (_a = shortcutMousetraps.get(s)) === null || _a === void 0 ? void 0 : _a.unbind(s.keys, s.action); });
          onChanged();
      };
      return () => {
          const [state, setState] = react.useState(keys);
          react.useEffect(() => {
              callbacks.add(setState);
              return () => callbacks.delete(setState);
          });
          return react.useMemo(() => [state, addKeys, removeKeys], [state]);
      };
  };
  const useHotkeyState = createStateHook();

  const useHotkeys = (shortcuts) => {
      const [keys, addKeys, removeKeys] = useHotkeyState();
      react.useLayoutEffect(() => {
          if (shortcuts) {
              addKeys(shortcuts);
          }
          return () => {
              if (shortcuts) {
                  removeKeys(shortcuts);
              }
          };
      }, [shortcuts]);
      return react.useMemo(() => keys.reduce((agg, cur) => {
          if (!agg.find((a) => a.keys === cur.keys && a.ref && cur.ref)) {
              agg.push(cur);
          }
          return agg;
      }, []), [keys]);
  };

  exports.useHotkeyState = useHotkeyState;
  exports.useHotkeys = useHotkeys;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
